<?php


namespace SwaggerBake\Test\TestCase\Lib\Attribute;

use Cake\Routing\Router;
use Cake\Routing\RouteBuilder;
use Cake\TestSuite\TestCase;

use SwaggerBake\Lib\Model\ModelScanner;
use SwaggerBake\Lib\Route\RouteScanner;
use SwaggerBake\Lib\Configuration;
use SwaggerBake\Lib\Swagger;

class OpenApiRequestBodyTest extends TestCase
{
    /**
     * @var string[]
     */
    public $fixtures = [
        'plugin.SwaggerBake.Employees',
    ];

    private Router $router;

    private Configuration $config;

    public function setUp(): void
    {
        parent::setUp(); // TODO: Change the autogenerated stub
        $router = new Router();
        $router::scope('/', function (RouteBuilder $builder) {
            $builder->setExtensions(['json']);
            $builder->resources('Employees', [
                'map' => [
                    'customPost' => [
                        'action' => 'customPost',
                        'method' => 'POST',
                        'path' => 'custom-post'
                    ],
                ]
            ]);
            $builder->resources('RequestBody', [
                'map' => [
                    'textPlain' => [
                        'action' => 'textPlain',
                        'method' => 'POST',
                        'path' => 'text-plain'
                    ],
                    'multipleMimeTypes' => [
                        'action' => 'multipleMimeTypes',
                        'method' => 'POST',
                        'path' => 'multiple-mime-types'
                    ],
                    'useConfigDefaults' => [
                        'action' => 'useConfigDefaults',
                        'method' => 'POST',
                        'path' => 'use-config-defaults'
                    ],
                ]
            ]);
        });
        $this->router = $router;

        $this->config = new Configuration([
            'prefix' => '/',
            'yml' => '/config/swagger-bare-bones.yml',
            'json' => '/webroot/swagger.json',
            'webPath' => '/swagger.json',
            'hotReload' => false,
            'exceptionSchema' => 'Exception',
            'requestAccepts' => ['application/x-www-form-urlencoded','application/xml','application/json'],
            'responseContentTypes' => ['application/json'],
            'namespaces' => [
                'controllers' => ['\SwaggerBakeTest\App\\'],
                'entities' => ['\SwaggerBakeTest\App\\'],
                'tables' => ['\SwaggerBakeTest\App\\'],
            ]
        ], SWAGGER_BAKE_TEST_APP);
    }

    public function test_open_api_request_body(): void
    {
        $routeScanner = new RouteScanner($this->router, $this->config);

        $swagger = new Swagger(new ModelScanner($routeScanner, $this->config));
        $arr = json_decode($swagger->toString(), true);

        $operation = $arr['paths']['/employees/custom-post']['post'];
        $body = $operation['requestBody'];

        $this->assertEquals('Hello', $body['description']);
        $this->assertTrue($body['required']);
        $this->assertCount(1, $body['content']['application/x-www-form-urlencoded']['schema']['properties']);
    }

    public function test_request_body_content(): void
    {
        $cakeRoute = new RouteScanner($this->router, $this->config);

        $swagger = new Swagger(new ModelScanner($cakeRoute, $this->config));
        $arr = json_decode($swagger->toString(), true);

        $operation = $arr['paths']['/request-body/text-plain']['post'];

        $this->assertArrayHasKey('schema', $operation['requestBody']['content']['text/plain']);
    }

    public function test_multiple_mime_types(): void
    {
        $cakeRoute = new RouteScanner($this->router, $this->config);

        $swagger = new Swagger(new ModelScanner($cakeRoute, $this->config));
        $arr = json_decode($swagger->toString(), true);

        $operation = $arr['paths']['/request-body/multiple-mime-types']['post'];

        $this->assertCount(2, $operation['requestBody']['content']);
    }

    public function test_use_mime_types_from_config(): void
    {
        $cakeRoute = new RouteScanner($this->router, $this->config);

        $swagger = new Swagger(new ModelScanner($cakeRoute, $this->config));
        $arr = json_decode($swagger->toString(), true);

        $operation = $arr['paths']['/request-body/use-config-defaults']['post'];

        $this->assertCount(3, $operation['requestBody']['content']);
    }
}