<?php

namespace SwaggerBake\Test\TestCase\Lib\Operation;

use Cake\Http\Exception\InternalErrorException;
use Cake\Routing\RouteBuilder;
use Cake\Routing\Router;
use Cake\TestSuite\TestCase;
use phpDocumentor\Reflection\DocBlockFactory;
use SwaggerBake\Lib\Configuration;
use SwaggerBake\Lib\Model\ModelScanner;
use SwaggerBake\Lib\OpenApi\Operation;
use SwaggerBake\Lib\Operation\OperationDocBlock;
use SwaggerBake\Lib\Route\RouteScanner;
use SwaggerBake\Lib\Swagger;

class OperationDocBlockTest extends TestCase
{
    /**
     * @var string[]
     */
    public $fixtures = [
        'plugin.SwaggerBake.Employees',
    ];

    private Router $router;

    private array $config;

    private Operation $operation;

    public function __construct(?string $name = null, array $data = [], $dataName = '')
    {
        parent::__construct($name, $data, $dataName);
        $this->operation = (new Operation('hello', 'get'));
    }

    public function setUp(): void
    {
        parent::setUp(); // TODO: Change the autogenerated stub
        $router = new Router();
        $router::scope('/', function (RouteBuilder $builder) {
            $builder->setExtensions(['json']);
            $builder->resources('Employees', [
                'only' => ['create']
            ]);
        });
        $this->router = $router;

        $this->config = [
            'prefix' => '/',
            'yml' => '/config/swagger-bare-bones.yml',
            'json' => '/webroot/swagger.json',
            'webPath' => '/swagger.json',
            'hotReload' => false,
            'exceptionSchema' => 'Exception',
            'requestAccepts' => ['application/x-www-form-urlencoded'],
            'responseContentTypes' => ['application/json'],
            'namespaces' => [
                'controllers' => ['\SwaggerBakeTest\App\\'],
                'entities' => ['\SwaggerBakeTest\App\\'],
                'tables' => ['\SwaggerBakeTest\App\\'],
            ]
        ];
    }

    /**
     * @throws InternalErrorException
     */
    public function test()
    {
        $config = new Configuration($this->config, SWAGGER_BAKE_TEST_APP);
        $cakeRoute = new RouteScanner($this->router, $config);
        $cakeModels = new ModelScanner($cakeRoute, $config);
        $swagger = new Swagger($cakeModels);

        $block = <<<EOT
/** 
 * @see http://www.cakephp.org CakePHP
 * @deprecated 
 */
EOT;
        $docBlock = DocBlockFactory::createInstance()->create($block);
        $operation = (new OperationDocBlock($swagger, $config, $this->operation, $docBlock))->getOperation();
        $doc = $operation->getExternalDocs();

        $this->assertEquals('CakePHP', $doc->getDescription());
        $this->assertEquals('http://www.cakephp.org', $doc->getUrl());
        $this->assertTrue($operation->isDeprecated());
    }

    /**
     * @throws InternalErrorException|\ReflectionException
     */
    public function test_without_description()
    {
        $config = new Configuration($this->config, SWAGGER_BAKE_TEST_APP);
        $cakeRoute = new RouteScanner($this->router, $config);
        $cakeModels = new ModelScanner($cakeRoute, $config);
        $swagger = new Swagger($cakeModels);

        $block = <<<EOT
/** 
 * @see http://www.cakephp.org
 * @deprecated 
 */
EOT;
        $docBlock = DocBlockFactory::createInstance()->create($block);
        $operation = (new OperationDocBlock($swagger, $config, $this->operation, $docBlock))->getOperation();
        $doc = $operation->getExternalDocs();

        $this->assertEquals('', $doc->getDescription());
        $this->assertEquals('http://www.cakephp.org', $doc->getUrl());
        $this->assertTrue($operation->isDeprecated());
    }

    /**
     * @throws \ReflectionException
     */
    public function test_invalid_url(): void
    {
        $config = new Configuration($this->config, SWAGGER_BAKE_TEST_APP);
        $cakeRoute = new RouteScanner($this->router, $config);
        $cakeModels = new ModelScanner($cakeRoute, $config);
        $swagger = new Swagger($cakeModels);

        $block = <<<EOT
/** 
 * @see htt:/www.cakephp.org
 * @deprecated 
 */
EOT;
        $docBlock = DocBlockFactory::createInstance()->create($block);
        $operation = (new OperationDocBlock($swagger, $config, $this->operation, $docBlock))->getOperation();
        $this->assertNull($operation->getExternalDocs());
    }
}